from .db import *
from .sakura import Sakura

class Client:
    def __init__(self, username: str, password: str, mongo: str):
        self.sakura_username = username
        self.sakura_password = password
        self.mongoURI = mongo
        self.sakura = Sakura(self.sakura_username, self.sakura_password, self.mongoURI)
        
    # SEND MESSAGE
    def sendMessage(self, uid: int, char_id: str, prompt: str):
        try:
            char_id_data = getChatId(uid, self.mongoURI)
            if char_id_data is None:
                # If no chat exists, create a new chat and store it in the database
                # print("Chat id does not exist, creating new chat")
                chat_data = self.sakura.create_chat(char_id, prompt)
                insertChat(chat_data['chat_id'], uid, char_id, self.mongoURI)
                return chat_data
            if char_id_data['char_id'] is not None and char_id_data['char_id'] != char_id:
                # If char id is changed, create a new chat and update it in the database with the new char id
                # print("Char id changed")
                new_char_id = char_id
                chat_data = self.sakura.create_chat(new_char_id, prompt)
                updateChatandCharId(uid, chat_data['chat_id'], new_char_id, self.mongoURI)
                return chat_data
            else:
                # If a chat exists, continue the chat
                # print("Chat id exists")
                chat_data = self.sakura.continue_chat(char_id_data['chat_id'], prompt)
                return chat_data

        except Exception as e:
            raise RuntimeError(f"An error occurred while sending the message: {str(e)}")

    # GET SELFIE
    def get_selfie(self, uid: int):
        try:
            chat_id = getChatId(uid, self.mongoURI)['chat_id']
            if chat_id:
                selfie = self.sakura.get_selfie(chat_id)
                return selfie
            else:
                raise RuntimeError("No chat found for the user. Cannot get the selfie.")
        except Exception as e:
            raise RuntimeError("An error occurred while getting the selfie: {}".format(str(e)))
    
    # DELETE DATABASE
    def delete_db(self):
        try:
            return delete_sakura_database(self.mongoURI)
        except Exception as e:
            raise RuntimeError("An error occurred while deleting the database: {}".format(str(e)))
        


from pymongo import MongoClient

# Function to insert a new chat
def insertChat(chat_id, user_id, char_id, mongoURI):
    client = MongoClient(mongoURI)
    db = client['SAKURA']

    # Define the chat collection
    chat_collection = db['Chat']
    # Insert the chat data into the collection
    chat_collection.insert_one({'chat_id': chat_id, 'user_id': user_id, 'char_id': char_id})
    return True

# Function to update chat_id and char_id of existing user_id
def updateChatandCharId(user_id, chat_id, char_id, mongoURI):
    client = MongoClient(mongoURI)
    db = client['SAKURA']

    # Define the chat collection
    chat_collection = db['Chat']

    # Update the chat data by user id
    chat_collection.update_one({'user_id': user_id}, {'$set': {'chat_id': chat_id, 'char_id': char_id}})
    return True

# Function to get a chat id by user id
def getChatId(user_id, mongoURI):
    client = MongoClient(mongoURI)
    db = client['SAKURA']

    # Define the chat collection
    chat_collection = db['Chat']

    # Find the chat data by user id
    chat = chat_collection.find_one({'user_id': user_id})
    if chat:
        return {
            'chat_id': chat['chat_id'],
            'char_id': chat['char_id']
        }
    else:
        return None

def delete_sakura_database(mongoURI):
    client = MongoClient(mongoURI)
    client.drop_database('SAKURA')
    print("SAKURA database deleted successfully.")
    return True


from os import error
import requests
from bs4 import BeautifulSoup
from datetime import datetime

class Sakura:
    def __init__(self, sakura_username, sakura_password, mongoURI):
        self.sakura_username = sakura_username
        self.sakura_password = sakura_password
        self.mongoURI = mongoURI
        self.baseUrl = "sakura.fm"
        
    # AUTHENTICATE
    def authenticate(self, jwt):
        headers = {
            'Content-Type': 'application/json',
            'Referer': f'https://www.{self.baseUrl}/',
            'Origin': f'https://www.{self.baseUrl}',
            'Authorization': f'Bearer {jwt}'
        }
        return headers


    # JWT
    def get_jwt(self):
        try:
            form_data = {
                'identifier': self.sakura_username,
                'password': self.sakura_password,
                'strategy': 'password'
            }
        
            headers = {
                'Content-Type': 'application/x-www-form-urlencoded',
                'Referer': f'https://www.{self.baseUrl}/',
                'Origin': f'https://www.{self.baseUrl}',
            }
        
            response = requests.post(f'https://clerk.{self.baseUrl}/v1/client/sign_ins?_clerk_js_version=5.1.1', data=form_data, headers=headers)
            response.raise_for_status()
            data = response.json()
            client_uat = data['client']['sessions'][0]['user']['last_sign_in_at']
            client_uat = int(str(client_uat)[:10])
            
            return {'jwt': data['client']['sessions'][0]['last_active_token']['jwt'], 'client_uat': client_uat}
        except:
            raise error('Failed to authenticate. Please ensure thta your username and password are correct.')


    # CONTINUE CHAT
    def continue_chat(self, chat_id, prompt):
        req_data = {
            "action": {
                "type": "append",
                "content": prompt
            },
            "context": {
                "chatId": chat_id,
                "locale": "en"
            }
        }

        jwt_data = self.get_jwt()
        jwt = jwt_data['jwt']
        headers = self.authenticate(jwt)

        response = requests.post(f'https://api.{self.baseUrl}/api/chat', json=req_data, headers=headers)
        data = response.json()
        reply = data['messages'][-1]
        return {'chat_id': chat_id,'reply': reply['content']}


    # CREATE CHAT
    def create_chat(self, char_id, prompt):
        # Get character start text
        response = requests.post(f'https://www.{self.baseUrl}/chat/{char_id}')
        
        soup = BeautifulSoup(response.text, 'html.parser')
        button = soup.find('div', class_='text-muted-foreground line-clamp-5')

        if button is None:
            raise ValueError("Button with the specified class not found")

        button_text = button.text.strip()
        char_start_text = button_text.replace('"', '')
        
        # Create chat with prompt
        req_data = {
            "action": {
                "type": "append",
                "content": prompt
            },
            "context": {
                "characterId": char_id,
                "messages": [
                    {
                        "role": "assistant",
                        "content": char_start_text,
                        "type": "text"
                    }
                ],
                "locale": "en"
            }
        }

        jwt_data = self.get_jwt()
        jwt = jwt_data['jwt']
        headers = self.authenticate(jwt)

        response = requests.post(f'https://api.{self.baseUrl}/api/chat', json=req_data, headers=headers)
        data = response.json()
        return {'chat_id': data['chatId'], 'reply': data['messages'][-1]['content']}


    # GET SELFIE
    def get_selfie(self, chat_id):
        jwt_data = self.get_jwt()
        jwt = jwt_data['jwt']
        headers = self.authenticate(jwt)

        req_data = {
            "action": {
                "type": "generate-image",
                "variant": "selfie"
            },
            "context": {
                "chatId": chat_id,
                "locale": "en"
            }
        }

        response = requests.post(f'https://api.{self.baseUrl}/api/chat', json=req_data, headers=headers)
        data = response.json()
        reply = data['messages'][-1]
        return reply['uri']




from setuptools import setup, find_packages

VERSION = "0.0.9"

def get_long_description():
    with open("README.md", encoding="UTF-8") as f:
        long_description = f.read()
        return long_description

setup(
    name='sakura-fm',
    version=VERSION,
    packages=find_packages(),
    include_package_data=True,
    long_description_content_type='text/markdown',
    author='Aditya',
    author_email='adityaraj6311@gmail.com',
    url="https://github.com/awesome-Tofu/sakura_fm",
    description='A simple scraper package for chatting with bots from sakura.fm',
    long_description=get_long_description(),
    install_requires=[
        'requests',
        'beautifulsoup4',
        'pymongo'
    ],
    keywords=['sakura', 'sakura.fm', 'sakura scraper', 'sakurai'],
    entry_points="""
        [console_scripts]
        sakura=sakura.sakura:main
    """
)